@@ -63,8 +63,10 @@ public class ClassFile {
     boolean methSynth;
     boolean methDepr;
     String methSigAttr;
-    VisibilityAnnotationAttr methAnnotAttr;
-    ParameterVisibilityAnnotationAttr methParamAnnotAttr;
+    VisibilityAnnotationAttr methAnnotAttrVis;
+    VisibilityAnnotationAttr methAnnotAttrInvis;
+    ParameterVisibilityAnnotationAttr methParamAnnotAttrVis;
+    ParameterVisibilityAnnotationAttr methParamAnnotAttrInvis;
     ElemValPair methAnnotDef;
     
     int line_label_count, line_num;
@@ -202,21 +204,27 @@ public class ClassFile {
         }
     }
     
-    void addClassAnnotAttr(Object res){
+    void addClassAnnotAttrVisible(Object res){
         if (res != null){
-            class_env.setClassAnnotAttr((VisibilityAnnotationAttr)res);
+            class_env.setClassAnnotAttrVis((VisibilityAnnotationAttr)res);
         }
     }
     
-    void addField(short access, String name, String sig, Object value, Object dep_attr, Object sig_attr, Object vis_annot_attr){
-        addField(access, name, sig, value, null, dep_attr, sig_attr, vis_annot_attr);
+    void addClassAnnotAttrInvisible(Object res){
+        if (res != null){
+            class_env.setClassAnnotAttrInvis((VisibilityAnnotationAttr)res);
+        }
+    }
+    
+    void addField(short access, String name, String sig, Object value, Object dep_attr, Object sig_attr, Object vis_annot_attr, Object vis_annot_attr2){
+        addField(access, name, sig, value, null, dep_attr, sig_attr, vis_annot_attr, vis_annot_attr2);
     }
 
     //
     // called by the .field directive
     //
     void addField(short access, String name,
-                                String sig, Object value, String synth, Object dep_attr, Object sig_attr, Object vis_annot_attr) {
+                                String sig, Object value, String synth, Object dep_attr, Object sig_attr, Object vis_annot_attr, Object vis_annot_attr2) {
 
 
         if (value == null) {
@@ -234,7 +242,18 @@ public class ClassFile {
             currentField.addSignatureAttr(new SignatureAttr((String)sig_attr));
         }
         if (vis_annot_attr != null){
-            currentField.addVisibilityAnnotationAttr((VisibilityAnnotationAttr)vis_annot_attr);
+            VisibilityAnnotationAttr attribute = (VisibilityAnnotationAttr)vis_annot_attr;
+            if(attribute.getKind().equals("RuntimeVisible"))
+            	currentField.addVisibilityAnnotationAttrVis(attribute);
+            else
+            	currentField.addVisibilityAnnotationAttrInvis(attribute);
+        }
+        if (vis_annot_attr2 != null){
+            VisibilityAnnotationAttr attribute = (VisibilityAnnotationAttr)vis_annot_attr2;
+            if(attribute.getKind().equals("RuntimeVisible"))
+            	currentField.addVisibilityAnnotationAttrVis(attribute);
+            else
+            	currentField.addVisibilityAnnotationAttrInvis(attribute);
         }
             
 	    /*currentField =
@@ -302,8 +321,10 @@ public class ClassFile {
         methSynth = false;
         methDepr = false;
         methSigAttr = null;
-        methAnnotAttr = null;
-        methParamAnnotAttr = null;
+        methAnnotAttrVis = null;
+        methAnnotAttrInvis = null;
+        methParamAnnotAttrVis = null;
+        methParamAnnotAttrInvis = null;
         methAnnotDef = null;
     }
 
@@ -344,11 +365,17 @@ public class ClassFile {
         if (methSigAttr != null){ 
             currentMethod.addSignatureAttr(new SignatureAttr(methSigAttr));
         }
-        if (methAnnotAttr != null){ 
-            currentMethod.addAnnotationAttr(methAnnotAttr);
+        if (methAnnotAttrVis != null){ 
+            currentMethod.addVisAnnotationAttr(methAnnotAttrVis);
         }
-        if (methParamAnnotAttr != null){ 
-            currentMethod.addParamAnnotationAttr(methParamAnnotAttr);
+        if (methAnnotAttrInvis != null){ 
+            currentMethod.addInvisAnnotationAttr(methAnnotAttrInvis);
+        }
+        if (methParamAnnotAttrVis != null){ 
+            currentMethod.addVisParamAnnotationAttr(methParamAnnotAttrVis);
+        }
+        if (methParamAnnotAttrInvis != null){ 
+            currentMethod.addInvisParamAnnotationAttr(methParamAnnotAttrInvis);
         }
         if (methAnnotDef != null){ 
             methAnnotDef.setNoName();
@@ -370,8 +397,8 @@ public class ClassFile {
 	    methSynth   = false;
         methDepr    = false;
         methSigAttr = null;
-        methAnnotAttr = null;
-        methParamAnnotAttr = null;
+        methAnnotAttrVis = null;
+        methParamAnnotAttrVis = null;
         methAnnotDef = null;
 
 
@@ -860,14 +887,22 @@ public class ClassFile {
         class_env.addEnclMethAttr(new EnclMethAttr(cls, meth, sig));
     }
    
-    void addMethAnnotAttr(Object attr){
-        methAnnotAttr = (VisibilityAnnotationAttr)attr;
+    void addMethAnnotAttrVisible(Object attr){
+		methAnnotAttrVis = (VisibilityAnnotationAttr)attr;
+    }
+
+    void addMethAnnotAttrInvisible(Object attr){
+		methAnnotAttrInvis = (VisibilityAnnotationAttr)attr;
+    }
+
+    void addMethParamAnnotAttrVisible(Object attr){
+        methParamAnnotAttrVis = (ParameterVisibilityAnnotationAttr)attr;
     }
    
-    void addMethParamAnnotAttr(Object attr){
-        methParamAnnotAttr = (ParameterVisibilityAnnotationAttr)attr;
+    void addMethParamAnnotAttrInvisible(Object attr){
+        methParamAnnotAttrInvis = (ParameterVisibilityAnnotationAttr)attr;
     }
-   
+
     void addMethAnnotDefault(Object attr){
         methAnnotDef = (ElemValPair)attr;
     }
